@@ -24,6 +24,7 @@ CGI::CGI() {
     char buf[1024 + 1];
     const char *cpstr;
     char *pstr;
+    const char *http_accept_language;
 
     (*this).clear();
     str.clear();
@@ -42,7 +43,7 @@ CGI::CGI() {
             if ((cpstr = getenv("QUERY_STRING")) != NULL) {
                 str = cpstr;
             }
-        } else if (!strcasecmp(buf, "POST") || !strcasecmp(buf, "PUT")) {
+        } else if (!strcasecmp(buf, "POST") || !strcasecmp(buf, "PUT") || !strcasecmp(buf, "PATCH")) {
             /* POST, PUT Method */
             if ((cpstr = getenv("CONTENT_LENGTH")) != NULL) {
                 int len, getlen, getlen_pre;
@@ -298,10 +299,16 @@ CGI::CGI() {
             esc_tmp_filename.clear();
             delete [] boundary;
 
-        } else if (cpstr != NULL && (strcasestr(cpstr, "xml") || strcasestr(cpstr, "json"))) {
+        } else if (cpstr != NULL && strcasestr(cpstr, "xml")) {
             // XMLHttpRequest
             str.useAsText();
             (*this).requestData["content"] = str;
+        } else if (cpstr != NULL && strcasestr(cpstr, "json")) {
+            // XMLHttpRequest (JSON)
+            JSONReader reader;
+            str.useAsText();
+            (*this).requestData["content"] = str;
+            reader.parse(str, (*this).requestJSON);
         } else {
             // normal request
             char *p, *prev_key, *prev_value;
@@ -374,6 +381,18 @@ CGI::CGI() {
         }
     }
 
+    // HTTP_ACCEPT_LANGUAGE is like "ja,zh-cn;q=0.8,en-us;q=0.3"
+    http_accept_language = getenv("HTTP_ACCEPT_LANGUAGE");
+    (*this).primary_lang[0] = '\0';
+    if (http_accept_language != NULL) {
+        strncpy((*this).primary_lang, http_accept_language, sizeof((*this).primary_lang) - 1);
+        (*this).primary_lang[sizeof((*this).primary_lang) - 1] = '\0';
+        pstr = strchr((*this).primary_lang, ',');
+        if (pstr != NULL) *pstr = '\0';
+        pstr = strchr((*this).primary_lang, ';');
+        if (pstr != NULL) *pstr = '\0';
+    }
+
     str.clear();
 }
 
@@ -399,17 +418,17 @@ bool CGI::clear() {
 }
 
 
-/*! Delete instance of CGI.
- *  @param  key  key of CGI parameter
- *  @return value of key
+/*! Get request value.
+ * @param  key  key of FCGI parameter
+ * @return value of key
  */
 String& CGI::getValue(const String &key) {
     return (*this).requestData[key];
 }
 
 
-/*! Delete instance of CGI.
- *  @param  key  key of CGI parameter
+/*! Get request value.
+ *  @param  key  key of FCGI parameter
  *  @return value of key
  */
 String& CGI::getValue(const char *key) {
@@ -417,6 +436,15 @@ String& CGI::getValue(const char *key) {
 }
 
 
+/*! Get request JSON.
+ *  @param void
+ *  @return JSONValue object of request
+ */
+const JSONValue& CGI::getRequestJSON() const {
+    return (*this).requestJSON;
+}
+
+
 /*! Get environment variable's value
  *  @param  key  key of Environment variable
  *  @return value of key
@@ -435,6 +463,15 @@ const char *CGI::getEnv(const char *key) const {
 }
 
 
+/*! Get browser primary language
+ *  @param void
+ *  return language id, ex. "en", "ja", ..
+ */
+const char *CGI::getPrimaryLanguage() const {
+    return (const char *)(*this).primary_lang;
+}
+
+
 /*! Delete instance of CGI.
  *  @param  key  key of CGI parameter
  *  @return value of key
@@ -516,7 +553,11 @@ bool CGI::putHeader() {
  */
 bool CGI::putContent(const String &content, long size) {
     if (size < 0) {
-        size = content.len();
+        if (content.isBinary()) {
+            size = content.binaryLength();
+        } else {
+            size = content.len();
+        }
     }
 
     return (*this).putContent(content.c_str(), size);
