@@ -29,6 +29,7 @@ FCGI::FCGI(FCGX_Request *request) {
     char buf[1024 + 1];
     const char *cpstr;
     char *pstr;
+    const char *http_accept_language;
 
     (*this).clear();
     str.clear();
@@ -50,7 +51,7 @@ FCGI::FCGI(FCGX_Request *request) {
             if ((cpstr = getenv("QUERY_STRING")) != NULL) {
                 str = cpstr;
             }
-        } else if (!strcasecmp(buf, "POST") || !strcasecmp(buf, "PUT")) {
+        } else if (!strcasecmp(buf, "POST") || !strcasecmp(buf, "PUT") || !strcasecmp(buf, "PATCH")) {
             /* POST, PUT Method */
             if ((cpstr = getenv("CONTENT_LENGTH")) != NULL) {
                 int len, tmplen, getlen, getlen_pre;
@@ -310,10 +311,16 @@ FCGI::FCGI(FCGX_Request *request) {
             esc_tmp_filename.clear();
             delete [] boundary;
 
-        } else if (cpstr != NULL && (strcasestr(cpstr, "xml") || strcasestr(cpstr, "json"))) {
+        } else if (cpstr != NULL && strcasestr(cpstr, "xml")) {
             // XMLHttpRequest
             str.useAsText();
             (*this).requestData["content"] = str;
+        } else if (cpstr != NULL && strcasestr(cpstr, "json")) {
+            // XMLHttpRequest (JSON)
+            JSONReader reader;
+            str.useAsText();
+            (*this).requestData["content"] = str;
+            reader.parse(str, (*this).requestJSON);
         } else {
             // normal request
             char *p, *prev_key, *prev_value;
@@ -383,6 +390,18 @@ FCGI::FCGI(FCGX_Request *request) {
         }
     }
 
+    // HTTP_ACCEPT_LANGUAGE is like "ja,zh-cn;q=0.8,en-us;q=0.3"
+    http_accept_language = getenv("HTTP_ACCEPT_LANGUAGE");
+    (*this).primary_lang[0] = '\0';
+    if (http_accept_language != NULL) {
+        strncpy((*this).primary_lang, http_accept_language, sizeof((*this).primary_lang) - 1);
+        (*this).primary_lang[sizeof((*this).primary_lang) - 1] = '\0';
+        pstr = strchr((*this).primary_lang, ',');
+        if (pstr != NULL) *pstr = '\0';
+        pstr = strchr((*this).primary_lang, ';');
+        if (pstr != NULL) *pstr = '\0';
+    }
+
     str.clear();
 }
 
@@ -408,7 +427,7 @@ bool FCGI::clear() {
 }
 
 
-/*! Delete instance of FCGI.
+/*! Get request value.
  *  @param  key  key of FCGI parameter
  *  @return value of key
  */
@@ -417,7 +436,7 @@ String& FCGI::getValue(const String &key) {
 }
 
 
-/*! Delete instance of FCGI.
+/*! Get request value.
  *  @param  key  key of FCGI parameter
  *  @return value of key
  */
@@ -426,6 +445,15 @@ String& FCGI::getValue(const char *key) {
 }
 
 
+/*! Get request JSON.
+ *  @param void
+ *  @return JSONValue object of request
+ */
+const JSONValue& FCGI::getRequestJSON() const {
+    return (*this).requestJSON;
+}
+
+
 /*! Get environment variable's value
  *  @param  key  key of Environment variable
  *  @return value of key
@@ -444,6 +472,15 @@ const char *FCGI::getEnv(const char *key) const {
 }
 
 
+/*! Get browser primary language
+ *  @param void
+ *  return language id, ex. "en", "ja", ..
+ */
+const char *FCGI::getPrimaryLanguage() const {
+    return (const char *)(*this).primary_lang;
+}
+
+
 /*! Delete instance of FCGI.
  *  @param  key  key of FCGI parameter
  *  @return value of key
@@ -527,7 +564,11 @@ bool FCGI::putHeader() {
  */
 bool FCGI::putContent(const String &content, long size) {
     if (size < 0) {
-        size = content.len();
+        if (content.isBinary()) {
+            size = content.binaryLength();
+        } else {
+            size = content.len();
+        }
     }
 
     return (*this).putContent(content.c_str(), size);
@@ -585,8 +626,9 @@ bool FCGI::putError(const char *err_str, long size) {
         size = strlen(err_str);
     }
 
-    fwrite(err_str, (size_t)size, 1, stderr);
+    //fwrite(err_str, (size_t)size, 1, stderr);
     FCGX_PutStr(err_str, (int)size, (*this).request->err);
+    FCGX_FFlush((*this).request->err);
 
     return true;
 }
