@@ -599,7 +599,7 @@ int test5() {
  */
 int test6() {
     // Declare Strings and Keys
-    String html, disp_html1, disp_html2, disp_html3;
+    String html, disp_html1, disp_html2, disp_html3, disp_html4, disp_html5, disp_html6, disp_html7;
     Keys html_keys;
 
     // Set Values
@@ -616,6 +616,27 @@ int test6() {
     html = "<html><body>&($KEY1?$KEY2:\"KEY1 is \\\"NOT\\\" set\"::html);</body></html>";
     disp_html3 = html.evalText(html_keys);
 
+    html_keys["KEY1"] = "on";
+    html_keys["KEY2"] = "on";
+    html = "<html><body>&($KEY1==$KEY2?$KEY2:\"not match\"::html);</body></html>";
+    disp_html4 = html.evalText(html_keys);
+
+    html_keys["KEY1"] = "";
+    html_keys["KEY2"] = "on";
+    html = "{{#if $KEY1}};key1{{#elif $KEY2}}key2{{#else}}else{{#endif}}";
+    disp_html5 = html.evalText(html_keys);
+
+    html_keys["KEY1"] = "";
+    html_keys["KEY2"] = "on";
+    html = "{{#if $KEY1==\"on\"}}key1{{#elif $KEY2==\"on\"}}key2{{#else}}else{{#endif}}";
+    disp_html6 = html.evalText(html_keys);
+
+    html_keys["KEY1"] = "";
+    html_keys["KEY2"] = "on";
+    html_keys["KEY3"] = "off";
+    html = "{{#if $KEY1==\"on\"}}key1{{#elif $KEY2==\"on\"}}{{#if $KEY3==\"off\"}}key3{{#else}}key2{{#endif}}{{#else}}else{{#endif}}";
+    disp_html7 = html.evalText(html_keys);
+
     // Expected Results
     // disp_html1 = <html><body>abc</body></html>
     // disp_html2 = <html><body>xyz&amp;XYZ</body></html>
@@ -636,6 +657,26 @@ int test6() {
         return -1;
     }
 
+    if (strcmp(disp_html4.c_str(), "<html><body>on</body></html>") != 0) {
+        fprintf(stderr, "Error: Test6 #4\n");
+        return -1;
+    }
+
+    if (strcmp(disp_html5.c_str(), "key2") != 0) {
+        fprintf(stderr, "Error: Test6 #5\n");
+        return -1;
+    }
+
+    if (strcmp(disp_html6.c_str(), "key2") != 0) {
+        fprintf(stderr, "Error: Test6 #6\n");
+        return -1;
+    }
+
+    if (strcmp(disp_html7.c_str(), "key3") != 0) {
+        fprintf(stderr, "Error: Test6 #7\n");
+        return -1;
+    }
+
     // Clear Allocated Memories (option)
     html.clear();
     disp_html1.clear();
